/////////////////////////////////////////////////////////////////////////////
// Name:        ligature.h
// Author:      Donald Byrd
// Created:     2016
// Copyright (c) Authors and others. All rights reserved.
/////////////////////////////////////////////////////////////////////////////

#ifndef __VRV_LIGATURE_H__
#define __VRV_LIGATURE_H__

#include "atts_shared.h"
#include "drawinginterface.h"
#include "durationinterface.h"
#include "layerelement.h"

namespace vrv {

//----------------------------------------------------------------------------
// Ligature
//----------------------------------------------------------------------------

/**
 * This class represents a collection of notes in the same layer with successive
 * onset times, as used in mensural notation.
 * A ligature is contained in a layer.
 * It contains notes.
 */

class Ligature : public LayerElement,
                 public ObjectListInterface,
                 public DurationInterface,
                 public AttStems,
                 public AttStemsCmn,
                 public AttTiepresent {
public:
    /**
     * @name Constructors, destructors, reset and class name methods
     * Reset method resets all attribute classes
     */
    ///@{
    Ligature();
    virtual ~Ligature();
    virtual void Reset();
    virtual std::string GetClassName() const { return "Ligature"; }
    virtual ClassId GetClassId() const { return LIGATURE; }
    ///@}

    /** Override the method since alignment is required */
    virtual bool HasToBeAligned() const { return true; }

    /**
     * Add an element (only notes are supported) to a ligature.
     */
    virtual void AddChild(Object *object);

    virtual void FilterList(ListOfObjects *childlist);

    /**
     * Return information about the note's position in the ligature ??
     */
    ///@{
    /** Return 0 if the note is the middle note, -1 if below it and 1 if above */
    int PositionInLigature(Note *note);
    ///@}

    //----------//
    // Functors //
    //----------//

protected:
    /**
     * Clear the m_clusters vector and delete all the objects.
     */
    void ClearClusters();

public:
};

} // namespace vrv

#endif
